import React from "react";
import { useDispatch, useSelector } from "react-redux";
export function useLoadingStatus(identifier = "global") {
    return useSelector((state) => state.loading[identifier] > 0);
}
/**
 * For actions like:
 * *foo(a: number, b: string, c: boolean): SagaIterator {..}
 *
 * useModuleAction(foo) will return:
 * (a: number, b: string, c: boolean) => void;
 *
 * useModuleAction(foo, 100) will return:
 * (b: string, c: boolean) => void;
 *
 * useModuleAction(foo, 100, "") will return:
 * (c: boolean) => void;
 *
 * useModuleAction(foo, 100, "", true) will return:
 * () => void;
 */
export function useModuleAction(actionCreator, ...deps) {
    const dispatch = useDispatch();
    return React.useCallback((...args) => dispatch(actionCreator(...deps, ...args)), [dispatch, actionCreator, ...deps]);
}
/**
 * For actions like:
 * *foo(data: {key: number}): SagaIterator {..}
 *
 * useModuleObjectAction(foo, "key") will return:
 * (objectValue: number) => void;
 */
export function useModuleObjectAction(actionCreator, objectKey) {
    const dispatch = useDispatch();
    return React.useCallback((objectValue) => dispatch(actionCreator({ [objectKey]: objectValue })), [dispatch, actionCreator, objectKey]);
}
//# sourceMappingURL=hooks.js.map